/* 
Electronic Technologies for Art II
et4a.roberttwomey.com | rtwomey@ucsd.edu

ble-button

Reads from builting SW1 (IO12) and publishes to service.
Use a BLE scanner (such as nRF Connect) or
this p5js sketch: https://editor.p5js.org/robert.twomey/sketches/8wQxLraNL

NOTE: change your BLEDevice name from "MyESP32" to something of your own.

Adapted from ESP32 Examples -> BLE -> Server
And NYU ITP https://itpnyu.github.io/p5ble-website/docs/read-one-char-callback

*/

#include <BLEDevice.h>
#include <BLEUtils.h>
#include <BLEServer.h>

// See the following for generating UUIDs:
// https://www.uuidgenerator.net/

#define SERVICE_UUID        "4fafc201-1fb5-459e-8fcc-c5c9c331914b"
#define BUTTON_CHARACTERISTIC_UUID "19B10012-E8F2-537E-4F6C-D104768A1214"

const int ledPin = 18; // set ledPin to on-board LED2
const int buttonPin = 12; // set buttonPin to BTN1
int sensorPin = A0;    // select the input pin for the potentiometer

// create switch characteristic and allow remote device to read and write
//BLEByteCharacteristic ledCharacteristic("19B10011-E8F2-537E-4F6C-D104768A1214", BLERead | BLEWrite);
// create button characteristic and allow remote device to get notifications
// BLEIntCharacteristic buttonCharacteristic("19B10012-E8F2-537E-4F6C-D104768A1214", BLERead | BLENotify);

BLECharacteristic *pCharacteristic;

void setup() {
  pinMode(ledPin, OUTPUT); // use the LED as an output
  pinMode(buttonPin, INPUT); // use button pin as an input

  Serial.begin(115200);
  Serial.println("Starting BLE work!");

  BLEDevice::init("MyESP32"); // change this name to a custom name
  BLEServer *pServer = BLEDevice::createServer();
  BLEService *pService = pServer->createService(SERVICE_UUID);
  pCharacteristic =
    pService->createCharacteristic(BUTTON_CHARACTERISTIC_UUID, BLECharacteristic::PROPERTY_READ | BLECharacteristic::PROPERTY_NOTIFY);
    // pService->createCharacteristic(CHARACTERISTIC_UUID, BLECharacteristic::PROPERTY_READ | BLECharacteristic::PROPERTY_WRITE);

  char txString[8];
  dtostrf(0, 1, 2, txString);

  pCharacteristic->setValue(txString); // set value to 0
  pService->start();
  // BLEAdvertising *pAdvertising = pServer->getAdvertising();  // this still is working for backward compatibility
  BLEAdvertising *pAdvertising = BLEDevice::getAdvertising();
  pAdvertising->addServiceUUID(SERVICE_UUID);
  pAdvertising->setScanResponse(true);
  pAdvertising->setMinPreferred(0x06);  // functions that help with iPhone connections issue
  pAdvertising->setMinPreferred(0x12);
  BLEDevice::startAdvertising();
  Serial.println("Characteristic defined! Now you can read it in your phone!");
}

void loop() {
  // put your main code here, to run repeatedly:
  delay(2000);

  // read the value from an analog sensor:
  // int sensorValue = analogRead(sensorPin);
  // char txString[8];
  // dtostrf(sensorValue/4, 1, 2, txString);

  // read value from digital button
  // char txString[8];
  // dtostrf(digitalRead(buttonPin), 1, 2, txString);
  // Serial.println(txString);
  // publish to characteristic
  // pCharacteristic->setValue(txString);

  int btn = digitalRead(buttonPin);

  Serial.println(btn);
  pCharacteristic->setValue(btn);

}
